package org.elsys.manytoone.junit;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

public class ManyToOneRelationTest {

	private ManyToOneRelation<String, String> r;

	@Before
	public void setUp() {
		r = new ManyToOneRelation<String, String>();
		r.connect("s1", "t1");
	}

	@Test
	public void testEqualsTargets() {
		// r:s1->t1
		// r:s1->t2
		// r2:s1->t2
		// r2:s1->t3
		r.connect("s1", "t2");

		ManyToOneRelation<String, String> r2 = new ManyToOneRelation<String, String>();
		r2.connect("s1", "t2");
		r2.connect("s1", "t3");

		assertFalse(r.equals(r2));
	}

	@Test
	public void testEquals() {
		// r:s1->t1
		// r:s1->t2
		// r2:s1->t1
		// r2:s1->t2
		// r3:s3->t1
		// r3:s3->t2
		r.connect("s1", "t2");

		ManyToOneRelation<String, String> r2 = new ManyToOneRelation<String, String>();
		r2.connect("s1", "t1");
		r2.connect("s1", "t2");

		ManyToOneRelation<String, String> r3 = new ManyToOneRelation<String, String>();
		r3.connect("s3", "t1");
		r3.connect("s3", "t2");

		assertEquals(r, r2);
		assertFalse(r.equals(r3));
	}

	@Test
	public void tesetContainsTarget() {
		r.connect("s2", "t2");

		assertTrue(r.containsTarget("t2"));
		assertFalse(r.containsTarget("t3"));
	}

	@Test
	public void testContainsSource() {
		r.connect("s2", "t2");

		assertTrue(r.containsSource("s1"));
		assertFalse(r.containsSource("s3"));
	}

	@Test
	public void testDisconnectTarget() {
		r.connect("s1", "t2");

		r.disconnectTarget("t1");
		assertEquals(1, r.getTargets("s1").size());
		assertEquals("t2", r.getTargets("s1").get(0));
	}

	@Test
	public void testDisconnectSource() {
		r.disconnectSource("s1");

		assertNull(r.getTargets("s1"));
	}

	@Test
	public void testChangeSource() {
		r.connect("s2", "t1");

		assertNull(r.getTargets("s1"));
		assertEquals(1, r.getTargets("s2").size());
		assertEquals("t1", r.getTargets("s2").get(0));
	}

	@Test
	public void testGetManyTargetsContent() {
		r.connect("s1", "t2");

		List<String> result = r.getTargets("s1");
		assertEquals(2, result.size());
		assertEquals("t1", result.get(0));
		assertEquals("t2", result.get(1));
	}

}
